var AWS = require("aws-sdk");

var region = "<リージョン名>";
var endpoint = "https://dynamodb." + region + ".amazonaws.com";
var table = "Notes";

var m_dynamoDB = null;
var m_dynamodbDoc = null;

function open() {
	if (m_dynamoDB === null) {
		console.log('open');
		m_dynamoDB = new AWS.DynamoDB();
		m_dynamoDB.config.update({
		  region: region,
		  endpoint: endpoint
		});
		m_dynamodbDoc = new AWS.DynamoDB.DocumentClient({service:m_dynamoDB});
	}
}

exports.getNoteList = getNoteList;

function getNoteList(userId, callback) {
	open();
	var params = {
		TableName: table,
		KeyConditionExpression: "user_id = :userId",
		ExpressionAttributeValues: {
			":userId": userId
		}
	};
	m_dynamodbDoc.query(params, function(err, data) {
		if (err) {
			console.log("failed to query.:", JSON.stringify(err, null, 2));
			callback(false, err);
		} else {
			console.log("getNoteList:");
			var notes = new Array();
			data.Items.forEach(function(item) {
				console.log(" -", JSON.stringify(item));
				var note = {
					user_id: item.user_id,
					date_time: item.date_time,
					text: item.text
				};
				notes.push(note);
			});
			callback(true, notes);
		}
	});
}

exports.updateNote = updateNote;

function updateNote(item, callback) {
	open();
	var params = {
		TableName : table,
		Item : {
			user_id: item.user_id,
			date_time: item.date_time,
			text: item.text
		}
	};

	m_dynamodbDoc.put(params, function(err, data) {
		if (err) {
			console.error("failed to update item.:", JSON.stringify(err, null, 2));
			callback(false, err);
		} else {
			console.log("Put item:", JSON.stringify(params.Item, null, 2));
			callback(true, params.Item);
		}
	});
}

exports.deleteNote = deleteNote;

function deleteNote(data, callback) {
	open();
	var dataList= [];
	dataList.push(data);
	deleteItems(dataList, function(isSuccess, result) {
		if (isSuccess) {
			callback(true, result[0]);
		} else {
			callback(false, result);
		}
	});
}

exports.clearAll = clearAll;

function clearAll(userId, callback) {
	open();
	console.log("clearAll ", userId);
	var params = {
		TableName: table,
		KeyConditionExpression: "user_id = :userId",
		ExpressionAttributeValues: {
			":userId": userId
		}
	};
	m_dynamodbDoc.query(params, function(err, data) {
		var notes = new Array();
		if (err) {
			console.log("failed to query.:", JSON.stringify(err, null, 2));
			callback(false, err);
		} else {
			deleteItems(data.Items, callback);
		}
	});
}

function deleteItems(items, callback) {
	params = {
		RequestItems: {}
	};
	var deletedItems = [];
	params.RequestItems[table] = [];
	console.log("delete items:");
	items.forEach(function(item) {
		console.log(" -", JSON.stringify(item));
		var request = {
			DeleteRequest: {
				Key: {
					user_id: item.user_id,
					date_time: item.date_time
				}
			}
		};
		params.RequestItems[table].push(request);
		deletedItems.push({
			user_id: item.user_id,
			date_time: item.date_time
		});
	});
	console.log("delete params:", JSON.stringify(params, null, 2));
	m_dynamodbDoc.batchWrite(params, function(err, data) {
		if (err) {
			console.log("failed to batchWriteItem.:", JSON.stringify(err, null, 2));
			callback(false, err);
		} else {
			console.log("batchWriteItem succeeded.:", JSON.stringify(deletedItems, null, 2));
			callback(true, deletedItems);
		}
	});
}
